var CaseSummary = function()
{
	var cs = {};
	var debug = 0;
	var home;
	var TRUNCATE_SIZE = [-1, 30, 20, 15];
	var defaultColClass = null;
	var chainedItems = [];

	cs.initPage = function()
	{
		Events.addEventHandler('showConfidential', 'change', cs.showConfidentialOnChange);
		Events.addEventHandler('columnBalance', 'change', cs.columnBalance);
		Events.doOnEnter('caseName', cs.caseSummary);
		Buttons.addClickHandler('btnCaseSummary', cs.caseSummary);
		Buttons.addClickHandler('printButton', cs.goPrint);
		Buttons.addClickHandler('close', App.closeWindow);
		
		home = Data.get('page-data', 'home-reggie');
		var caseName = Data.get('page-data', 'case-name');
		var isAdmin = Data.int('page-data', 'admin');
		var isPatientCurator = Data.int('page-data', 'patient-curator');

		var url = '../CaseSummary.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo';
		url += '&caseName='+encodeURIComponent(caseName);
		Wizard.showLoadingAnimation('Loading information about case: ' + Strings.encodeTags(caseName));
		Wizard.asyncJsonRequest(url, cs.caseInfoLoaded);
	}
	
	cs.caseInfoLoaded = function(response)
	{
		Doc.show('all-info');
		
		var caseName = Data.get('page-data', 'case-name');
		var caseInfo = response.caseInfo;
		var site = response.site;
		var patient = response.patient;
		cs.initSections(response.sections);
		var consentOk = true;
		
		var maxItemsInLeftCol = 1;
		var maxItemsInRightCol = 1;

		if (caseInfo)
		{
			// Use returned case name, as it may have been found from personal number
			if (!patient || patient.allCases.indexOf(caseName) == -1)
			{
				caseName = caseInfo.name;
				Data.set('page-data', 'case-name', caseName);
				// Update header text to use the potentially changed case name
				if (Doc.element('path-case-summary'))
				{
					Doc.element('path-case-summary').innerHTML = 'Case summary - ' + Strings.encodeTags(caseName);
				}
			}
			
			try
			{
				var consentOk = !caseInfo.consent || caseInfo.consent == 'Yes';
				cs.addColumn('case.name', cs.asLink('SAMPLE', caseInfo));
				cs.addColumn('case.registrationDate', cs.asDate(caseInfo.registrationDate));
				cs.addColumn('case.site', site ? site.name : null);
				cs.addColumn('case.consent', cs.asConsent(caseInfo.consent, cs.asDate(caseInfo.consentDate)));
				cs.addColumn('case.laterality', cs.warnIfMissing(Strings.encodeTags(caseInfo.laterality)));
				cs.addColumn('case.frozenTissueDate', cs.asDate(caseInfo.frozenTissueDate));
				cs.addColumn('case.comment', Strings.encodeTags(caseInfo.comment));
			}
			catch (e)
			{
				cs.logError('case-info', e);
			}
		}
		else
		{
			cs.addColumn('case.name', cs.asNoInfo('No case information has been registered'));
			Doc.hide('case-details');
		}

		if (patient)
		{
			try
			{
				cs.addColumn('patient.name', cs.asLink('BIOSOURCE', patient));
				cs.addColumn('patient.registrationDate', cs.asDate(patient.registrationDate));
				cs.addColumn('patient.personalNumber', cs.warnIfMissing(Strings.encodeTags(patient.personalNumber)));
				cs.addColumn('patient.allFirstNames', cs.warnIfMissing(Strings.encodeTags(patient.allFirstNames)));
				cs.addColumn('patient.familyName', cs.warnIfMissing(Strings.encodeTags(patient.familyName)));
				cs.addColumn('patient.gender', Strings.encodeTags(patient.gender));
				cs.addColumn('patient.allCases', cs.asCaseList(patient.allCases, caseName));
			}
			catch (e)
			{
				cs.logError('patient-info', e);
			}
		}
		else
		{
			cs.addColumn('patient.name', cs.asNoInfo('No patient information has been registered'));
			Doc.hide('patient-details');
		}

		var blood = response.blood;
		if (blood && blood.length > 0)
		{
			maxItemsInLeftCol = Math.max(maxItemsInLeftCol, blood.length);
			for (var i = 0; i < blood.length; i++)
			{
				try
				{
					var b = blood[i];
					cs.addColumn('blood.name', cs.asLink('SAMPLE', b));
					cs.addColumn('blood.registrationDate', cs.asDate(b.registrationDate));
					cs.addColumn('blood.consent', cs.asConsent(b.consent, cs.asDate(b.consentDate)));
					cs.addColumn('blood.bloodSample', Strings.encodeTags(b.bloodSample));
					cs.addColumn('blood.samplingDate', cs.warnIfMissing(cs.asDateTime(b.samplingDate)));
					cs.addColumn('blood.freezerDate', cs.warnIfMissing(cs.asDateTime(b.freezerDate, b.samplingDate)));
					cs.addColumn('blood.serum', Strings.encodeTags(b.serum));
					cs.addColumn('blood.comment', Strings.encodeTags(b.comment));
				}
				catch (e)
				{
					cs.logError('blood-info', e);
				}
			}
		}
		else
		{
			cs.addColumn('blood.name', cs.asNoInfo('No blood information has been registered'));
			Doc.hide('blood-details');
		}

		// Blood DNA
		var bdna = response.bdna;
		if (bdna && bdna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(bdna.length-1, TRUNCATE_SIZE.length)];
			//maxItemsInLeftCol = Math.max(maxItemsInLeftCol, bdna.length);
			for (var i = 0; i < bdna.length; i++)
			{
				try
				{
					var d = bdna[i];
					cs.addColumn('bdna.name', cs.asLink('EXTRACT', d));
					cs.addColumn('bdna.registrationDate', cs.asDate(d.registrationDate));
					cs.addColumn('bdna.storageBox', cs.asBioPlateLocation(d.bioWell));
					cs.addColumn('bdna.remainingQuantity', cs.asQuantity(d.remainingQuantity, ' µg'));
					cs.addColumn('bdna.originalQuantity', cs.asQuantity(d.originalQuantity, ' µg'));
					cs.addColumn('bdna.ndConc', cs.asQuantity(d.ndConc, ' ng/µl'));
					cs.addColumn('bdna.comment', cs.truncate(d.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('bdna-info', e);
				}
			}
		}
		else
		{
			cs.addColumn('bdna.name', cs.asNoInfo('No Blood DNA information has been registered'));
			Doc.hide('bdna-details');
		}
		
		var specimen = response.specimen;
		var noSpecimen = response.noSpecimen;
		var numSpecimenOrNoSpecimen = 0;
		var yellowPrefixes = {};
		if (specimen && specimen.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(specimen.length-1, TRUNCATE_SIZE.length)];
			maxItemsInLeftCol = Math.max(maxItemsInLeftCol, specimen.length);
			for (var i = 0; i < specimen.length; i++)
			{
				try
				{
					var s = specimen[i];
					yellowPrefixes[s.name] = s.YellowLabel != null ? 'yellow-specimen' : null;
					defaultColClass = cs.getDefaultColClass(s, yellowPrefixes);
					cs.addColumn('specimen.name', cs.asLink('SAMPLE', s, 0, true));
					cs.addColumn('specimen.donotuse', cs.asDoNotUse(s));
					cs.addColumn('specimen.registrationDate', cs.asDate(s.registrationDate));
					cs.addColumn('specimen.storageBox', cs.asBioPlateLocation(s.bioWell));
					cs.addColumn('specimen.laterality', cs.warnIfMissing(Strings.encodeTags(s.laterality)));
					cs.addColumn('specimen.samplingDate', cs.warnIfMissing(cs.asDateTime(s.samplingDate)));
					cs.addColumn('specimen.rnaLaterDate', cs.warnIfMissing(cs.asDateTime(s.rnaLaterDate, s.samplingDate)));
					cs.addColumn('specimen.pad', cs.warnIfMissing(Strings.encodeTags(s.pad)));
					cs.addColumn('specimen.biopsyType', s.biopsyType);
					cs.addColumn('specimen.remainingQuantity', cs.asQuantity(s.remainingQuantity, ' mg', 1000));
					cs.addColumn('specimen.originalQuantity', cs.asQuantity(s.originalQuantity, ' mg', 1000));
					cs.addColumn('specimen.nofPieces', s.nofPieces);
					cs.addColumn('specimen.partitionComment', cs.truncate(s.partitionComment, truncateAt));
					cs.addColumn('specimen.comment', cs.truncate(s.comment, truncateAt));
					numSpecimenOrNoSpecimen++;
				}
				catch (e)
				{
					cs.logError('specimen-info', e);
				}
			}
			defaultColClass = null;
			if (!noSpecimen || noSpecimen.length == 0)
			{
				Doc.hide('nospecimen-info');
			}
		}
		if (noSpecimen && noSpecimen.length > 0)
		{
			maxItemsInLeftCol = Math.max(maxItemsInLeftCol, noSpecimen.length);
			for (var i = 0; i < noSpecimen.length; i++)
			{
				try
				{
					var s = noSpecimen[i];
					cs.addColumn('nospecimen.name', cs.asLink('SAMPLE', s, 0, true));
					cs.addColumn('nospecimen.registrationDate', cs.asDate(s.registrationDate));
					cs.addColumn('nospecimen.laterality', cs.warnIfMissing(Strings.encodeTags(s.laterality)));
					cs.addColumn('nospecimen.samplingDate', cs.warnIfMissing(cs.asDate(s.samplingDate)));
					cs.addColumn('nospecimen.pad', cs.warnIfMissing(Strings.encodeTags(s.pad)));
					cs.addColumn('nospecimen.reasonIfNoSpecimen', Strings.encodeTags(s.reasonIfNoSpecimen));
					cs.addColumn('nospecimen.comment', cs.truncate(s.comment, truncateAt));
					numSpecimenOrNoSpecimen++;
				}
				catch (e)
				{
					cs.logError('nospecimen-info', e);
				}
			}
			defaultColClass = null;
			if (!specimen || specimen.length == 0)
			{
				cs.addColumn('specimen.name', cs.asNoInfo('No specimen received; see below for details'));
				Doc.hide('specimen-details');
			}
		}
		if (numSpecimenOrNoSpecimen == 0)
		{
			cs.addColumn('specimen.name', cs.asNoInfo('No specimen information has been registered'));
			Doc.hide('specimen-details');
			Doc.hide('nospecimen-info');
		}

		
		var histology = response.histology;
		if (histology && histology.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(histology.length-1, TRUNCATE_SIZE.length)];
			maxItemsInLeftCol = Math.max(maxItemsInLeftCol, histology.length);
			for (var i = 0; i < histology.length; i++)
			{
				try
				{
					var h = histology[i];
					defaultColClass = cs.getDefaultColClass(h, yellowPrefixes);
					cs.addColumn('histology.name', cs.asLink('SAMPLE', h, 0, true));
					cs.addColumn('histology.donotuse', cs.asDoNotUse(h));
					cs.addColumn('histology.registrationDate', cs.asDate(h.registrationDate));
					cs.addColumn('histology.partitionDate', cs.asDate(h.partitionDate));
					cs.addColumn('histology.usedQuantity', cs.asQuantity(h.originalQuantity, ' mg', 1000));
	
					var embedded = h.embedDate ? true : false;
					cs.addColumn('histology.embedded', embedded ? cs.asDate(h.embedDate) : 'No');
					cs.addColumn('histology.embedProtocol', embedded ? cs.asLink('PROTOCOL', h.embedProtocol, truncateAt) : null);
					cs.addColumn('histology.paraffinBlock', embedded ? cs.asBioPlateLocation(h.bioWell) : null);
					cs.addColumn('histology.storageBox', embedded ? null : cs.asBioPlateLocation(h.bioWell));
	
					var stained = h.stainDate ? true : false;
					var bestStain = stained ? h.bestStain : null;
					var scored = bestStain && bestStain.ScoreComplete != null ? true : false;
						
					cs.addColumn('histology.stained', stained ? cs.asDate(h.stainDate) : 'No');
					cs.addColumn('histology.nofSlides', stained ? h.numStains : null);
					cs.addColumn('histology.heGlass', bestStain ? cs.asBioPlateLocation(bestStain.bioWell) : null);
					cs.addColumn('histology.stainProtocol', stained ? cs.asLink('PROTOCOL', h.stainProtocol, truncateAt) : null);

					cs.addColumn('histology.scored', (scored ? (bestStain.ScoreComplete ? 'Complete' : 'In progress') : 'No')+cs.asFileLink(bestStain ? bestStain.image : null, 'picture.png'));
					cs.addColumn('histology.scoreInvasiveCancer', scored ? cs.asPercent(bestStain.ScoreInvasiveCancer) : null);
					cs.addColumn('histology.scoreInsituCancer', scored ? cs.asPercent(bestStain.ScoreInsituCancer) : null);
					cs.addColumn('histology.scoreLymphocytes', scored ? cs.asPercent(bestStain.ScoreLymphocytes) : null);
					cs.addColumn('histology.scoreNormal', scored ? cs.asPercent(bestStain.ScoreNormal) : null);
					cs.addColumn('histology.scoreStroma', scored ? cs.asPercent(bestStain.ScoreStroma) : null);
					cs.addColumn('histology.scoreFat', scored ? cs.asPercent(bestStain.ScoreFat) : null);
					cs.addColumn('histology.scoreComments', scored ? cs.truncate(bestStain.comments, truncateAt) : null);
				}
				catch (e)
				{
					cs.logError('histology-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('histology.name', cs.asNoInfo('No histology information has been registered'));
			Doc.hide('histology-details');
		}

		var lysate = response.lysate;
		if (lysate && lysate.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(lysate.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, lysate.length);
			for (var i = 0; i < lysate.length; i++)
			{
				try
				{
					var lys = lysate[i];
					defaultColClass = cs.getDefaultColClass(lys, yellowPrefixes);
					cs.addColumn('lysate.name', cs.asLink('EXTRACT', lys, 0, true));
					cs.addColumn('lysate.donotuse', cs.asDoNotUse(lys));
					cs.addColumn('lysate.registrationDate', cs.asDate(lys.registrationDate));
					cs.addColumn('lysate.partitionDate', cs.asDate(lys.partitionDate));
					cs.addColumn('lysate.usedQuantity', cs.asQuantity(lys.usedQuantity, ' mg', 1000));
					cs.addColumn('lysate.multiplePieces', lys.multiplePieces ? 'Yes' : 'No');
					cs.addColumn('lysate.comment', cs.truncate(lys.comment, truncateAt));
					
					var lysis = lys.lysisDate ? true : false;
					cs.addColumn('lysate.lysisDate', lysis ? cs.asDate(lys.lysisDate) : 'No');
					cs.addColumn('lysate.storageBox', lysis ? cs.asBioPlateLocation(lys.bioWell) : null);
					cs.addColumn('lysate.lysisProtocol', lysis ? cs.asLink('PROTOCOL', lys.lysisProtocol, truncateAt) : null);
					cs.addColumn('lysate.remainingQuantity', lysis ? cs.asQuantity(lys.remainingQuantity, ' µg') : null);
					cs.addColumn('lysate.originalQuantity', lysis ? cs.asQuantity(lys.originalQuantity, ' µg') : null);
				}
				catch (e)
				{
					cs.logError('lysate-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('lysate.name', cs.asNoInfo('No lysate information has been registered'));
			Doc.hide('lysate-details');
		}

		var rna = response.rna;
		if (rna && rna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(rna.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, rna.length);
			for (var i = 0; i < rna.length; i++)
			{
				try
				{
					var r = rna[i];
					defaultColClass = cs.getDefaultColClass(r, yellowPrefixes);
					cs.addColumn('rna.name', cs.asLink('EXTRACT', r, 0, true));
					cs.addColumn('rna.flag', r.flag ? cs.asFlagInfo(r.flag, 'flag.png') : null);
					cs.addColumn('rna.donotuse', cs.asDoNotUse(r));
					cs.addColumn('rna.registrationDate', cs.asDate(r.registrationDate));
					cs.addColumn('rna.storageBox', cs.asBioPlateLocation(r.bioWell));
					cs.addColumn('rna.usedQuantity', cs.asQuantity(r.usedQuantity, ' µg'));
					cs.addColumn('rna.extractionDate', cs.asDate(r.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.extractionDate, 'RNA extraction: ' + r.name, 'thermometer.png'));
					cs.addColumn('rna.extractionProtocol', cs.asLink('PROTOCOL', r.extractionProtocol, truncateAt));
					cs.addColumn('rna.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' µg'));
					cs.addColumn('rna.originalQuantity', cs.asQuantity(r.originalQuantity, ' µg'));
					cs.addColumn('rna.ndConc', cs.asQuantity(r.ndConc, ' ng/µl'));
					cs.addColumn('rna.comment', cs.truncate(r.comment, truncateAt));
					
					var hasQc = r.nofQc ? true : false;
					cs.addColumn('rna.qc', hasQc ? (r.qcDate ? cs.asDate(r.qcDate) : cs.asNoInfo('Waiting for Caliper...')) : 'No');
					cs.addColumn('rna.nofQc', hasQc ? r.nofQc : null);
					cs.addColumn('rna.qcProtocol', hasQc ? cs.asLink('PROTOCOL', r.qcProtocol, truncateAt) : null);
					cs.addColumn('rna.qcPlate', hasQc ? cs.asBioPlateLocation(r.qcPlate)+cs.asFileLink(r.qcPdf, 'pdffile.png') : null);
					cs.addColumn('rna.qcRqs', hasQc && r.qcRqs ? Reggie.formatNumber(r.qcRqs, null, 1) : null);
					cs.addColumn('rna.qcRin', hasQc && r.qcRin ? Reggie.formatNumber(r.qcRin, null, 1) : null);
					cs.addColumn('rna.qcComment', hasQc ? cs.truncate(r.qcComment, truncateAt) : null);
				}
				catch (e)
				{
					cs.logError('rna-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('rna.name', cs.asNoInfo('No RNA information has been registered'));
			Doc.hide('rna-details');
		}

		var dna = response.dna;
		if (dna && dna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(dna.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, dna.length);
			for (var i = 0; i < dna.length; i++)
			{
				try
				{
					var d = dna[i];
					defaultColClass = cs.getDefaultColClass(d, yellowPrefixes);
					cs.addColumn('dna.name', cs.asLink('EXTRACT', d, 0, true));
					cs.addColumn('dna.donotuse', cs.asDoNotUse(d));
					cs.addColumn('dna.registrationDate', cs.asDate(d.registrationDate));
					cs.addColumn('dna.storageBox', cs.asBioPlateLocation(d.bioWell));
					cs.addColumn('dna.extractionDate', cs.asDate(d.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', d.extractionDate, 'DNA extraction: ' + d.name, 'thermometer.png'));
					cs.addColumn('dna.extractionProtocol', cs.asLink('PROTOCOL', d.extractionProtocol, truncateAt));
					cs.addColumn('dna.remainingQuantity', cs.asQuantity(d.remainingQuantity, ' µg'));
					cs.addColumn('dna.originalQuantity', cs.asQuantity(d.originalQuantity, ' µg'));
					cs.addColumn('dna.ndConc', cs.asQuantity(d.ndConc, ' ng/µl'));
					cs.addColumn('dna.comment', cs.truncate(d.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('dna-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('dna.name', cs.asNoInfo('No DNA information has been registered'));
			Doc.hide('dna-details');
		}

		var ft = response.flowThrough;
		if (ft && ft.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(ft.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, ft.length);
			for (var i = 0; i < ft.length; i++)
			{
				try
				{
					var f = ft[i];
					defaultColClass = cs.getDefaultColClass(f, yellowPrefixes);
					cs.addColumn('ft.name', cs.asLink('EXTRACT', f, 0, true));
					cs.addColumn('ft.donotuse', cs.asDoNotUse(f));
					cs.addColumn('ft.registrationDate', cs.asDate(f.registrationDate));
					cs.addColumn('ft.storageBox', cs.asBioPlateLocation(f.bioWell));
					cs.addColumn('ft.extractionDate', cs.asDate(f.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', f.extractionDate, 'FlowThrough extraction: ' + f.name, 'thermometer.png'));
					cs.addColumn('ft.extractionProtocol', cs.asLink('PROTOCOL', f.extractionProtocol, truncateAt));
					cs.addColumn('ft.remainingQuantity', cs.asQuantity(f.remainingQuantity, ' µg'));
					cs.addColumn('ft.originalQuantity', cs.asQuantity(f.originalQuantity, ' µg'));
					cs.addColumn('ft.comment', cs.truncate(f.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('ft-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('ft.name', cs.asNoInfo('No FlowThrough information has been registered'));
			Doc.hide('ft-details');
		}

		var mrna = response.mrna;
		if (mrna && mrna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(mrna.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, mrna.length);
			for (var i = 0; i < mrna.length; i++)
			{
				try
				{
					var r = mrna[i];
					defaultColClass = cs.getDefaultColClass(r, yellowPrefixes);
					cs.addColumn('mrna.name', cs.asLink('EXTRACT', r, 0, true));
					cs.addColumn('mrna.donotuse', cs.asDoNotUse(r));
					cs.addColumn('mrna.registrationDate', cs.asDate(r.registrationDate));
					cs.addColumn('mrna.storageBox', cs.asBioPlateLocation(r.bioWell)+cs.asFileLink(r.platePdf, 'pdffile.png'));
					cs.addColumn('mrna.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
					cs.addColumn('mrna.cleanupDate', cs.asDate(r.cleanupDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.cleanupDate, 'mRNA cleanup: ' + r.name, 'thermometer.png'));
					cs.addColumn('mrna.usedQuantity', cs.asQuantity(r.usedQuantity, ' µg'));
					cs.addColumn('mrna.comment', cs.truncate(r.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('mrna-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('mrna.name', cs.asNoInfo('No mRNA information has been registered'));
			Doc.hide('mrna-details');
		}

		var cdna = response.cdna;
		if (cdna && cdna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(cdna.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, cdna.length);
			for (var i = 0; i < cdna.length; i++)
			{
				try
				{
					var r = cdna[i];
					defaultColClass = cs.getDefaultColClass(r, yellowPrefixes);
					cs.addColumn('cdna.name', cs.asLink('EXTRACT', r, 0, true));
					cs.addColumn('cdna.donotuse', cs.asDoNotUse(r));
					cs.addColumn('cdna.registrationDate', cs.asDate(r.registrationDate));
					cs.addColumn('cdna.storageBox', cs.asBioPlateLocation(r.bioWell));
					cs.addColumn('cdna.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
					cs.addColumn('cdna.synthesisDate', cs.asDate(r.synthesisDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.synthesisDate, 'cDNA synthesis: ' + r.name, 'thermometer.png'));
					cs.addColumn('cdna.comment', cs.truncate(r.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('cdna-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('cdna.name', cs.asNoInfo('No cDNA information has been registered'));
			Doc.hide('cdna-details');
		}
		
		var lib = response.lib;
		if (lib && lib.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(lib.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, lib.length);
			for (var i = 0; i < lib.length; i++)
			{
				try
				{
					var r = lib[i];
					defaultColClass = cs.getDefaultColClass(r, yellowPrefixes);
					cs.addColumn('lib.name', cs.asLink('EXTRACT', r, 0, true));
					cs.addColumn('lib.donotuse', cs.asDoNotUse(r));
					cs.addColumn('lib.pipeline', r.pipeline);
					cs.addColumn('lib.registrationDate', cs.asDate(r.registrationDate));
					cs.addColumn('lib.storageBox', cs.asBioPlateLocation(r.bioWell));
					cs.addColumn('lib.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
					var libDate = r.cleanupDate || r.neoPrepDate;
					cs.addColumn('lib.libDate', cs.asDate(libDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', libDate, 'Library date: ' + r.name, 'thermometer.png'));
					cs.addColumn('lib.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' ng', 0.001));
					cs.addColumn('lib.originalQuantity', cs.asQuantity(r.originalQuantity, ' ng', 0.001));
					cs.addColumn('lib.barcode.name', r.barcode.name);
					cs.addColumn('lib.size', cs.asQuantity(r.ca_size || r.library_size_est, ''));
					cs.addColumn('lib.molarity', cs.asQuantity(r.library_molarity_est, ' nM'));
					cs.addColumn('lib.adapterFrac', cs.asPercent(r.library_frac_adpt));
					cs.addColumn('lib.conc', cs.asQuantity(r.qubitconc || r.library_conc_est, ' ng/µl'));
					cs.addColumn('lib.comment', cs.truncate(r.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('lib-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('lib.name', cs.asNoInfo('No library information has been registered'));
			Doc.hide('lib-details');
		}

		var pooledLib = response.pooledlib;
		if (pooledLib && pooledLib.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(pooledLib.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, pooledLib.length);
			for (var i = 0; i < pooledLib.length; i++)
			{
				try
				{
					var r = pooledLib[i];
					cs.addColumn('pooledlib.name', cs.asLink('EXTRACT', r, 0, r.libNames));
					cs.addColumn('pooledlib.pipeline', r.pipeline);
					cs.addColumn('pooledlib.registrationDate', cs.asDate(r.registrationDate));
					cs.addColumn('pooledlib.poolDate', cs.asDate(r.poolDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.poolDate, 'Pooled library pooling: ' + r.name, 'thermometer.png'));
					cs.addColumn('pooledlib.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' ng', 0.001));
					cs.addColumn('pooledlib.originalQuantity', cs.asQuantity(r.originalQuantity, ' ng', 0.001));
					cs.addColumn('pooledlib.poolMolarity', cs.asQuantity(r.poolMolarity, ' nM'));
					cs.addColumn('pooledlib.poolConc', cs.asQuantity(r.poolConc, ' ng/µl'));
					cs.addColumn('pooledlib.comment', cs.truncate(r.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('pooledlib-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('pooledlib.name', cs.asNoInfo('No pooled library information has been registered'));
			Doc.hide('pooledlib-details');
		}

		var sequencingRuns = response.sequencingRun;
		if (sequencingRuns && sequencingRuns.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(sequencingRuns.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, sequencingRuns.length);
			for (var i = 0; i < sequencingRuns.length; i++)
			{
				try
				{
					var sr = sequencingRuns[i];
					var fc = sr.flowCell;
					cs.addColumn('sequencingRun.name', cs.asLink('DERIVEDBIOASSAY', sr, 0, sr.libNames));
					cs.addColumn('sequencingRun.flowCell', cs.asLink('PHYSICALBIOASSAY', fc));
					cs.addColumn('sequencingRun.pipeline', sr.pipeline);
					cs.addColumn('sequencingRun.result', (sr.result && sr.result != 'Successful') ? cs.asFailInfo(sr.result, 'error.png') : null);
					cs.addColumn('sequencingRun.clusterDate', cs.asDate(fc.clusterDate));
					cs.addColumn('sequencingRun.startDate', cs.asDate(sr.startDate));
					cs.addColumn('sequencingRun.endDate', cs.asDate(sr.endDate));
					cs.addColumn('sequencingRun.comment', cs.truncate(sr.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('sequencingRun-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('sequencingRun.name', cs.asNoInfo('No sequencing information has been registered'));
			Doc.hide('sequencingRun-details');
		}

		var mergedSequences = response.mergedSequences;
		if (mergedSequences && mergedSequences.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(mergedSequences.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, mergedSequences.length);
			for (var i = 0; i < mergedSequences.length; i++)
			{
				try
				{
					var m = mergedSequences[i];
					defaultColClass = cs.getDefaultColClass(m, yellowPrefixes);
					cs.addColumn('mergedSequences.name', cs.asLink('DERIVEDBIOASSAY', m, 0, true));
					cs.addColumn('mergedSequences.donotuse', cs.asDoNotUse(m));
					cs.addColumn('mergedSequences.pipeline', m.pipeline);
					var job = m.job;
					cs.addColumn('mergedSequences.date', cs.asJob(job, truncateAt, 'Demuxing'));
					cs.addColumn('mergedSequences.result', (m.result && m.result != 'Successful') ? cs.asFailInfo(m.result, 'error.png') : null);
					cs.addColumn('mergedSequences.pfReads', cs.asCount(m.PF_READS));
					cs.addColumn('mergedSequences.ptReads', cs.asCount(m.PT_READS));
					cs.addColumn('mergedSequences.adapterReads', cs.asCount(m.ADAPTER_READS));
					cs.addColumn('mergedSequences.comment', cs.truncate(m.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('mergedSequences-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('mergedSequences.name', cs.asNoInfo('No demux information has been registered'));
			Doc.hide('mergedSequences-details');
		}
		
		var alignedSequences = response.alignedSequences;
		if (alignedSequences && alignedSequences.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(alignedSequences.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, alignedSequences.length);
			for (var i = 0; i < alignedSequences.length; i++)
			{
				try
				{
					var a = alignedSequences[i];
					defaultColClass = cs.getDefaultColClass(a, yellowPrefixes);
					cs.addColumn('alignedSequences.name', cs.asLink('DERIVEDBIOASSAY', a, 0, true));
					cs.addColumn('alignedSequences.donotuse', cs.asDoNotUse(a));
					cs.addColumn('alignedSequences.pipeline', a.pipeline);
					var job = a.job;
					cs.addColumn('alignedSequences.software', cs.asLink('SOFTWARE', a.software, truncateAt));
					cs.addColumn('alignedSequences.date', cs.asJob(job, truncateAt, 'Aligning'));
					cs.addColumn('alignedSequences.result', (a.result && a.result != 'Successful') ? cs.asFailInfo(a.result, 'error.png') : null);
					cs.addColumn('alignedSequences.alignedPairs', cs.asCount(a.ALIGNED_PAIRS));
					cs.addColumn('alignedSequences.fractionDuplication', cs.asPercent(a.FRACTION_DUPLICATION, true));
					cs.addColumn('alignedSequences.genotypeQc', cs.asGenotypeQc(a));
					cs.addColumn('alignedSequences.comment', cs.truncate(a.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('alignedSequences-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('alignedSequences.name', cs.asNoInfo('No aligned information has been registered'));
			Doc.hide('alignedSequences-details');
		}

		var rawBioAssays = response.rawBioAssays;
		if (rawBioAssays && rawBioAssays.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(rawBioAssays.length-1, TRUNCATE_SIZE.length)];
			maxItemsInRightCol = Math.max(maxItemsInRightCol, rawBioAssays.length);
			for (var i = 0; i < rawBioAssays.length; i++)
			{
				try
				{
					var raw = rawBioAssays[i];
					defaultColClass = cs.getDefaultColClass(raw, yellowPrefixes);
					cs.addColumn('rawBioAssays.name', cs.asLink('RAWBIOASSAY', raw, 0, true));
					cs.addColumn('rawBioAssays.donotuse', cs.asDoNotUse(raw));
					cs.addColumn('rawBioAssays.pipeline', raw.pipeline);
					var job = raw.job;
					var geneReport = raw.geneReport;
					var pilotReport = raw.pilotReport;
					cs.addColumn('rawBioAssays.type', Strings.encodeTags(raw.rawDataType));
					cs.addColumn('rawBioAssays.date', cs.asJob(job, truncateAt));
					cs.addColumn('rawBioAssays.result', (raw.result && raw.result != 'Successful') ? cs.asFailInfo(raw.result, 'error.png') : null);
					cs.addColumn('rawBioAssays.geneReport', geneReport && geneReport.type == 'FILE' ? 'Yes '+cs.asFileLink(geneReport, 'pdffile.png') : 'No');
					cs.addColumn('rawBioAssays.pilotReport', pilotReport && pilotReport.type == 'FILE' ? 'Yes '+cs.asFileLink(pilotReport, 'pdffile.png') : 'No');
					cs.addColumn('rawBioAssays.comment', cs.truncate(a.comment, truncateAt));
				}
				catch (e)
				{
					cs.logError('rawBioAssays-info', e);
				}
			}
			defaultColClass = null;
		}
		else
		{
			cs.addColumn('rawBioAssays.name', cs.asNoInfo('No raw bioassay information has been registered'));
			Doc.hide('rawBioAssays-details');
		}
		
		var linkedItems = document.getElementsByClassName('linked-item');
		for (var itemNo = 0; itemNo < linkedItems.length; itemNo++)
		{
			Events.addEventHandler(linkedItems[itemNo], 'click', Items.itemOnClick);
		}
		
		var linkedFiles = document.getElementsByClassName('linked-file');
		for (var fileNo = 0; fileNo < linkedFiles.length; fileNo++)
		{
			Events.addEventHandler(linkedFiles[fileNo], 'click', Files.viewFileOnClick);
		}
		
		var linkedVcf = document.getElementsByClassName('vcf-link');
		for (var i = 0; i < linkedVcf.length; i++)
		{
			Events.addEventHandler(linkedVcf[i], 'click', cs.vcfLinkOnClick);
		}
		
		var linkedLabEnvs = document.getElementsByClassName('linked-labenv');
		for (var labEnvNo = 0; labEnvNo < linkedLabEnvs.length; labEnvNo++)
		{
			Events.addEventHandler(linkedLabEnvs[labEnvNo], 'click', LabEnvs.viewDataOnClick);
		}
		
		var linkedCases = document.getElementsByClassName('linked-case');
		for (var caseNo = 0; caseNo < linkedCases.length; caseNo++)
		{
			Events.addEventHandler(linkedCases[caseNo], 'click', cs.caseSummary);
		}
		
		chainedItems = document.getElementsByClassName('chained-item');
		for (var itemNo = 0; itemNo < chainedItems.length; itemNo++)
		{
			Events.addEventHandler(chainedItems[itemNo], 'click', cs.chainedItemOnClick);
		}
		
		if (maxItemsInRightCol >= Math.max(3, 2*maxItemsInLeftCol))
		{
			Doc.element('columnBalance').value = 4;
			Events.sendChangeEvent('columnBalance');
		}
		else if (maxItemsInLeftCol >= Math.max(3, 2*maxItemsInRightCol))
		{
			Doc.element('columnBalance').value = 2;
			Events.sendChangeEvent('columnBalance');
		}
		
		var highlight = Data.get('page-data', 'highlight')+'.';
		if (highlight)
		{
			var targetList = highlight.split(',');
			var rootItem = cs.highlightChainedItems(targetList, targetList[0]);
			if (rootItem)
			{
				Doc.addClass(rootItem.parentNode, 'rootitem');
			}
		}
	}
	
	cs.chainedItemOnClick = function(event)
	{
		// A possible list of 'chain-id' values that should be highlighted
		var targetList = Data.get(event.currentTarget, 'chain-id').split(',');
		cs.highlightChainedItems(targetList);
		Doc.addClass(event.currentTarget.parentNode, 'rootitem');
	}
	
	/**
		Highlight all items that share part of the name with the target
		that was clicked. Note that parent items have a shorter name and
		child items have a longer name so we need two tests.
	*/
	cs.highlightChainedItems = function(targetList, scrollTo)
	{
		// 'targetList' is a possible list of 'chain-id' values that should be highlighted
		// 'autoScroll' is used as a marker so that that only the first <td> within the same <table> is scrolled into view
		var autoScroll = new Date().getTime();
		var rootItem = null;
		for (var itemNo = 0; itemNo < chainedItems.length; itemNo++)
		{
			var item = chainedItems[itemNo];
			var chainId = Data.get(item, 'chain-id').split(',');
			var inChain = cs.hasCommonElement(targetList, chainId);
			Doc.addOrRemoveClass(item.parentNode, 'chained', inChain);
			Doc.removeClass(item.parentNode, 'rootitem');
			
			// This handles vertical scrolling to the root item
			if (scrollTo && chainId.indexOf(scrollTo) >= 0) 
			{
				item.parentNode.scrollIntoView();
				scrollTo = null;
				rootItem = item;
			}
			
			if (inChain)
			{
				// Try to scroll the highlighted item into view
				// We need to get the <td> aligned left or right in the <div> that is the 
				// parent to the <table>
				var td = item.parentNode;
				var tbl = td.parentNode.parentNode.parentNode; // Moving: td -> tr -> thead -> table
				var div = tbl.parentNode;
				if (div.autoScroll != autoScroll)
				{
					if (td.offsetLeft < div.scrollLeft) 
					{
						div.scrollLeft = td.offsetLeft;
						div.autoScroll = autoScroll;
					} 
					else if (td.offsetLeft + td.offsetWidth > div.scrollLeft + div.offsetWidth)
					{
						div.scrollLeft = td.offsetLeft + td.offsetWidth - div.offsetWidth;
						div.autoScroll = autoScroll;
					}
				}
			}
		}
		return rootItem;
	}
	
	/**
		Check if the two lists have at least a pair of elements
		were one is a substring of the other.
	*/
	cs.hasCommonElement = function(list1, list2)
	{
		for (var i1 = 0; i1 < list1.length; i1++)
		{
			var e1 = list1[i1];
			for (var i2 = 0; i2 < list2.length; i2++)
			{
				var e2 = list2[i2];
				if (e2.indexOf(e1) == 0 || e1.indexOf(e2) == 0) return true;
			}
		}
		return false;
	}
	
	cs.initSections = function(sections)
	{
		for (var s = 0; s < sections.length; s++)
		{
			var section = sections[s];

			var details = Doc.element(section.id+'-details');
			// If the details section is 'forcibly' hidden (eg. no items of that type), ignore this call
			if (details && details.style.display != 'none')
			{
				var headerDiv = Doc.element(section.id+'-header');
				if (headerDiv)
				{
					Doc.addClass(headerDiv, 'link');
					headerDiv.title = 'Hide/show details';
					headerDiv.sectionId = section.id;
					Events.addEventHandler(headerDiv, 'click', cs.toggleDetails);
				}
				
				var sectionDiv = Doc.element(section.id+'-info');
				Doc.addOrRemoveClass(sectionDiv, 'hide-details', section.hiddenDetails);
			}
		}
	}

	cs.toggleDetails = function(event)
	{
		var sectionId = event.currentTarget.sectionId;
		var details = Doc.element(sectionId+'-details');
		// If the details section is 'forcibly' hidden (eg. no items of that type), ignore this call
		if (details.style.display == 'none') return;
		
		// Add 'hide-details' class to main <div section-info> if it is visible
		var section = Doc.element(sectionId+'-info');
		var isVisible = section.className.indexOf('hide-details') == -1;
		Doc.addOrRemoveClass(section, 'hide-details', isVisible);

		// Save to BASE setting
		var url = '../CaseSummary.servlet?ID='+App.getSessionId();
		url += '&cmd=SetSectionVisibility';
		url += '&section='+sectionId+'&hidden='+(isVisible ? 1 : 0);
		Wizard.asyncJsonRequest(url, null, 'POST');
	}

	cs.getDefaultColClass = function(item, yellowPrefixes)
	{
		var defaultColClass = yellowPrefixes[item.name.substr(0,9)];
		if (item.DO_NOT_USE)
		{
			defaultColClass = 'donotuse ' + (defaultColClass || '');
		}
		return defaultColClass;
	}
	
	cs.addColumn = function(id, value, className)
	{
		var tr = Doc.element(id);
		var td = document.createElement('td');
		if (!className) className = defaultColClass;
		if (className) td.className = className;
		if (tr.hasAddedColumns)
		{
			td.className += ' extra-column';
			tr.childNodes[tr.childNodes.length-1].className += ' fixed-column';
		}
		tr.hasAddedColumns = true;
		if (value != null)
		{
			td.innerHTML = value == '' ? '&nbsp;' : value;
			if (tr.className.indexOf('dynamic-column') >= 0)
			{
				tr.style.display = 'table-row';
			}
		}
		else
		{
			td.innerHTML = '&nbsp;';
		}
		tr.appendChild(td);
	}

	cs.asLink = function(itemType, item, maxLength, chainable)
	{
		var link = '';
		if (item)
		{
			var name = cs.truncate(item.name, maxLength);
			var tooltip = item.DO_NOT_USE ? 'DoNotUse-'+Strings.encodeTags(item.DO_NOT_USE + ': ' + (item.DO_NOT_USE_COMMENT || '')) : '';
			if (item.id)
			{
				link = '<span class="link linked-item" title="'+tooltip+'" data-item-type="'+itemType+'" data-item-id="'+item.id+'" data-no-edit="'+(item.editable ? 0 : 1)+'">';
				link += name + '</span>';
			}
			else
			{
				link = name;
			}
			if (chainable) 
			{
				var chainId = item.name;
				if (chainable.length) chainId = chainable.join('.,');
				// NOTE! 'chain-id' is name of item + '.' to ensure correct matching of substrings ('foo.m2' is NOT in the same chain when starting at 'foo.m')
				link += '<img class="chained-item" data-chain-id="'+Strings.encodeTags(chainId)+'." title="Highlight all parent/child items" src="'+home+'/images/parent-child.png">';
			}
		}
		return link;
	}
	
	cs.asFileLink = function(file, icon)
	{
		var link = '';
		if (file)
		{
			link = '<span class="link linked-file" data-file-id="'+file.id+'"><img src="'+home+'/images/'+icon+'" style="margin-left: 2px;"></span>';
		}
		return link;
	}
	
	cs.asJob = function(job, maxLength, taskType)
	{
		var text = null;
		var tooltip = 'on ' + Strings.encodeTags(job.server);
		if (job.status == 'ERROR')
		{
			text = cs.asFailInfo(cs.truncate(job.statusMessage, maxLength), 'error.png');
		}
		else if (!job.started)
		{
			text = 'Waiting since ' + cs.asDateTime(job.scheduled);
		}
		else if (!job.ended)
		{
			text = (taskType || 'Running') + ' ('+cs.asPercent(job.percentComplete) + ' since ' + cs.asDateTime(job.started) + ')';
		}
		if (text)
		{
			text = '<span class="link linked-item" data-item-type="JOB" data-item-id="'+job.id+'" title="'+tooltip+'">' + text + '</span>';
		}
		else
		{
			text = cs.asDateTime(job.ended);
		}
		return text;
	}

	cs.asLabEnvLink = function(sensor, date, title, icon)
	{
		var link = '';
		// Check if BASE extension LabEnv is installed
		var homeLabEnv = Data.get('page-data', 'home-labenv');
		if (homeLabEnv)
		{
			if (date)
			{
				if (sensor)
				{
					// Link to pop-up window with lab environment data for the chosen sensor and date
					link = '<span class="link linked-labenv" data-sensor="'+sensor+'" data-date="'+date+'" data-title="'+title+'"><img src="'+home+'/images/'+icon+'"></span>';
				}
			}
		}
		return link;
	}

	cs.asCaseList = function(allCases, mainCase)
	{
		var html = '';
		for (var i = 0; i < allCases.length; i++)
		{
			var cse = allCases[i];
			var cse2 = Strings.encodeTags(cse);
			if (html != '') html += ', ';
			if (cse != mainCase)
			{
				html += '<span class="link linked-case" data-case-name="'+cse2+'" title="Show summary of case #'+cse2+'">'+cse2+'</span>';
			}
			else
			{
				html += '<b>' + cse2 + '</b>';
			}
		}
		return allCases.length > 1 ? html : null;
	}

	
	// Format value as a date
	cs.asDate = function(value)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = value.substr(0, 4) + '-' + value.substr(4, 2) + '-' + value.substr(6, 2);
		}
		return value;
	}

	// Format as date+time value
	cs.asDateTime = function(value, compareToDate)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = cs.asDate(value);
		}
		else if (value.length == 13)
		{
			// If the compareToDate is the same day as the 'value' date, skip the date and replace with white-space
			if (compareToDate && value.substr(0, 8) == compareToDate.substr(0, 8))
			{
				value = '<span class="invisible">'+cs.asDate(value.substr(0, 8)) + '</span> ' + value.substr(9, 2) + ':'+value.substr(11, 2)
			}
			else
			{
				value = cs.asDate(value.substr(0, 8)) + ' ' + value.substr(9, 2) + ':'+value.substr(11, 2);
			}
		}
		return value;
	}

	// Style the value as a quantity with (optional) unit
	cs.asQuantity = function(value, unit, scale)
	{
		var result = null;
		if (value != null)
		{
			if (scale) value = value / scale;
			result = Reggie.formatNumber(value, unit, 2);
		}
		return result;
	}
	
	// Style the value as a percentage value
	cs.asPercent = function(value, isFraction)
	{
		var result = '';
		if (value != null)
		{
			if (isFraction) value = value * 100;
			result = Reggie.formatNumber(value, '%', 1);
		}
		return result;
	}

	// Style the value as a count; 
	// values > 1 000 000 are given in millions
	// valus > 1000 are given in thousands
	cs.asCount = function(value)
	{
		var result = '';
		if (value != null)
		{
			if (value > 1000000) 
			{
				result = Reggie.formatNumber(value / 1000000, 'M', 1);
			}
			else if (value > 1000)
			{
				result = Reggie.formatNumber(value / 1000, 'k', 1);
			}
			else
			{
				result = value;
			}
		}
		return result;
	}

	// Style the message as a warning message
	cs.asWarning = function(message)
	{
		message = '<span class="warning">' + message + '</span>';
		return message;
	}

	// Generate a 'Missing' warning if the value is missing
	cs.warnIfMissing = function(value)
	{
		var message = value ? value : cs.asWarning('Missing');
		return message;
	}
	
	// Style the message as a 'no information' message
	cs.asNoInfo = function(message)
	{
		message = '<span class="no-info">' + message + '</span>';
		return message;
	}
	
	// Style the message as a 'flag information' message
	cs.asFlagInfo = function(message, icon)
	{
		message = '<span class="flag-info">' + message + '&nbsp;' + '<img src="'+home+'/images/'+icon+'"></span>';
		return message;
	}

	// Style the message as a 'DoNotUse information' message
	cs.asDoNotUse = function(item)
	{
		var message = null;
		if (item.DO_NOT_USE)
		{
			message = Strings.encodeTags(item.DO_NOT_USE + ': ' + (item.DO_NOT_USE_COMMENT || ''));
		}
		return message;
	}

	
	// Style the message as a 'fail information' message
	cs.asFailInfo = function(message, icon)
	{
		message = '<span class="fail-info">' + message + '&nbsp;' + '<img src="'+home+'/images/'+icon+'"></span>';
		return message;
	}
	
	cs.asConsent = function(consent, consentDate)
	{
		var warn = !consent || !consentDate || consent != 'Yes';
		var message;
		if (!consent)
		{
			message = 'Missing' + (consentDate ? ', ' + consentDate : '');
		}
		else
		{
			message = consent + ', ' + (consentDate ? consentDate : 'Missing date');
			
		}
		return warn ? cs.asWarning(message) : message;
	}
	
	// A biowell as the location
	cs.asBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = cs.asLink('BIOPLATE', plate)  + ' ' + well.location;
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = Strings.encodeTags(storage.name);
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;

			text = '<span class="more-info" title="' + tmp.join('; ')+'">' + text + ' </span>';
		}
		return text;
	}

	cs.asGenotypeQc = function(alignment)
	{
		var result = cs.asCount(alignment.QC_GENOTYPE_COUNT);
		if (alignment.QC_GENOTYPE_HET_PCT > 0)
		{
			result += ' (' + cs.asPercent(alignment.QC_GENOTYPE_HET_PCT, false) + ' HET)';
		}
		if (alignment.qcVcf)
		{
			result += '<span class="link vcf-link" data-file-id="'+alignment.qcVcf.id+'" data-item-id="'+alignment.id+'"><img src="'+home+'/images/vcf_file.png" style="margin-left: 2px;"></span>';
		}
		return result;
	}
	
	cs.truncate = function(value, maxLength)
	{
		if (!value) return value;
		if (maxLength > 2 && value.length > maxLength) 
		{
			var tmp = '<span class="truncated" title="'+value+'">'+Strings.encodeTags(value.substring(0, maxLength-2)) + '...</span>';
			value = tmp;
		}
		else
		{
			value = Strings.encodeTags(value);
		}
		return value;
	}

	cs.showConfidentialOnChange = function()
	{
		var frm = document.forms['reggie'];
		Doc.addOrRemoveClass('all-info', 'hide-confidential', !frm.showConfidential.checked);
	}

	cs.caseSummary = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = Data.get(event.currentTarget, 'case-name');
		if (!caseName)
		{
			caseName = frm.caseName.value;
			if (!caseName)
			{
				Forms.showNotification('caseName', 'Please enter a 7-digit case id', null, null, 'left');
				return;
			}
		}
		var url = 'case_summary.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(caseName);
		url += '&highlight='+encodeURIComponent(caseName);
		url += '&pageType='+Data.get('page-data', 'page-type');
		location.href = url;
	}

	cs.goPrint = function()
	{
		var caseName = Data.get('page-data', 'case-name');
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.<br>';
		printNote += ' You may have to <i>scale down</i> to fit everything on the width of the page.';
		Reggie.openPrintWindow('all-content', 'Case summary - ' + Strings.encodeTags(caseName), 'portrait', printNote, '../', 'case_summary.css');
	}

	
	cs.logError = function(section, err)
	{
		console.error(err);
		section = Doc.element(section);
		var div = document.createElement('div');
		div.className = 'messagecontainer error';
		div.innerHTML = err;
		section.parentNode.insertBefore(div, section.nextSibling);
	}
	
	cs.vcfLinkOnClick = function(event)
	{
		var fileId = Data.int(event.currentTarget, 'file-id');
		var itemId = Data.get(event.currentTarget, 'item-id');

		var url = home+'/analysis/view_genotypes.jsp?ID=' + App.getSessionId();
		url += '&fileId='+fileId;
		url += '&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewVcf', 750, 500);
	}
	
	cs.columnBalance = function(event)
	{
		Data.set('all-content', 'cbal', event.currentTarget.value);
	}
	
	return cs;
}();

var LabEnvs = function()
{
	var labenvs = {};

	/**
		Event handler that calls 'labenv.jar/reports/labenvironmentdatabaseeventpopup.jsp'
		when clicking on the attached target. The sensor id should be stored
		in attribute 'data-sensor', the date in 'data-date',
		and the title in 'data-title'.
	*/
	labenvs.viewDataOnClick = function(event)
	{
		var homeLabEnv = Data.get('page-data', 'home-labenv');
		var title = Data.get(event.currentTarget, 'title');
		var sensor = Data.get(event.currentTarget, 'sensor');
		var date = Data.get(event.currentTarget, 'date');
		var url = homeLabEnv;
		url += '/reports/labenvironmentdatabaseeventpopup.jsp';
		url += '?ID='+App.getSessionId();
		url += '&pageTitle='+encodeURIComponent(title);
		url += '&labSensorUrl='+encodeURIComponent(sensor);
		url += '&originDate='+encodeURIComponent(date);
		Dialogs.openPopup(url, title, 820, 768);
	}
	
	return labenvs;
}();

Doc.onLoad(CaseSummary.initPage);
